import os
import json
import logging
import pytest
import yaml
from tycho.model import System
from tycho.core import Tycho
from tycho.client import TychoClientFactory

logger = logging.getLogger (__name__)

def get_sample_spec (name):
    """ Load a docker-compose specification from our samples. """
    result = None
    d = os.path.dirname (__file__)
    sample_path = os.path.join (d, "..", "sample", name, "docker-compose.yaml")
    with open(sample_path, "r") as stream:
        result = yaml.load (stream)
    return result

def make_request ():
    """ Create a Tycho request object to test with. """
    request = {
        "name"   : "test",
        "principal": '{"username": "renci"}',
        "env"    : {},
        "system" : get_sample_spec ("jupyter-ds"),
        "services" : {
            "jupyter-datascience" : {
                "port"    : "8888",
                "clients" : [ "127.0.0.1" ]
            }
        }
    }
    return request

@pytest.fixture(scope='module')
def system_request ():
    return make_request ()
    
@pytest.fixture(scope='module')
def system(system_request):
    """ Create a Tycho request object to test with. """
    tycho = Tycho (backplane='kubernetes') 
    print (f"{json.dumps(system_request, indent=2)}")
    return tycho.parse (system_request)

@pytest.fixture(scope='module')
def client():
    return TychoClientFactory().get_client ()



import logging
import pytest
from tycho.test.lib import client
from tycho.test.lib import system_request
# https://medium.com/@yeraydiazdiaz/what-the-mock-cheatsheet-mocking-in-python-6a71db997832

@pytest.fixture(params=["data1", "data2"])
def volume_model_data(request):
    if request.param is "data1":
        data = [{
            "name": "nginx",
            "image": "sample/image:v1",
            "command": "some entrypoint",
            "env": [],
            "limits": [],
            "requests": {},
            "ports": [],
            "expose": [],
            "depends_on": [],
            "volumes": ["pvc://nfsrods/rods:/home/rods", "pvc://cloud-top:/home/shared"]
        }]
        return data
    if request.param is "data2":
        data = [
            {'container_name': 'nginx',
             'pvc_name': 'nfsrods',
             'volume_name': 'nfsrods',
             'path': '/home/rods',
             'subpath': 'rods'
             },
            {'container_name': 'nginx',
             'pvc_name': 'cloud-top',
             'volume_name': 'cloud-top',
             'path': '/home/shared',
             'subpath': ''
             }
        ]
        return data



import pytest
import sys
from tycho.model import Volumes

# Sample data from conftest.py
@pytest.mark.parametrize('volume_model_data', ['data1'], indirect=True)
def test_model_volume(volume_model_data):
    id_num = "333333333"
    volumes = Volumes(id_num, volume_model_data)
    data_volumes = volumes.process_volumes()
    data_volume_1 = data_volumes[0]
    assert data_volume_1["container_name"] == "nginx"
    assert data_volume_1["pvc_name"] == "nfsrods"
    assert data_volume_1["volume_name"] == "nfsrods"
    assert data_volume_1["path"] == "/home/rods"
    assert data_volume_1["subpath"] == "rods"
    data_volume_2 = data_volumes[1]
    assert data_volume_2["subpath"] == ''





class TychoException(Exception):
    def __init__(self, message, details=""):
        super().__init__(message)
        self.details = details

class StartException(Exception):
    def __init__(self, message, details=""):
        super().__init__(message, details)
class DeleteException(Exception):
    def __init__(self, message, details=""):
        super().__init__(message, details)
class ContextException(TychoException):
    def __init__(self, message, details=""):
        super().__init__(message, details)



import os
import json
import yaml
from tycho.model import System
from tycho.config import Config

def test_system_model (request):
    """ Test that the abstract model parses structures as we expect and puts the pieces
    where they belong. """
    print (f"{request.node.name}")
    system = System (**{
        "config"     : Config (),
        "name"       : "test",
        "principal"   : {"username": "renci"},
        "serviceAccount": "default",
        "containers" : [
            {
                "name"  : "nginx-container",
                "image" : "nginx:1.9.1",
                "limits" : [{
                    "cpus" : "0.5",
                    "memory" : "512M"
                }],
                "volumes": [],
            }
        ]
    }) 
    print(system.containers[0].limits)
    assert system.name.startswith ('test-')
    assert system.containers[0].name == 'nginx-container'
    assert system.containers[0].image == 'nginx:1.9.1'
    assert system.containers[0].limits['cpus'] == "0.5"
    assert system.containers[0].limits['memory'] == "512M"

def test_system_parser (request):
    """ Test parsing of a docker-compose into the standard model. """
    print (f"{request.node.name}")
    base_dir = os.path.dirname (os.path.dirname (__file__))
    spec_path = os.path.join (base_dir, "sample", "jupyter-ds", "docker-compose.yaml")
    with open (spec_path, "r") as stream:
        structure = yaml.load (stream)
        system =  System.parse (
            config = Config (),
            name   = "jupyter-ds",
            principal = '{"username": "renci"}',
            system = structure,
            serviceAccount = "default")

        print (f"{system}")
        assert system.name.startswith ('jupyter-ds')
        assert system.containers[0].name == 'jupyter-ds'
        assert system.containers[0].image == 'jupyter/datascience-notebook'
        assert system.containers[0].limits.cpus == '0.01'
        assert system.containers[0].limits.memory == '50M'
        assert system.containers[0].requests.cpus == '0.01'
        assert system.containers[0].requests.memory == '20M'
        assert system.containers[0].ports[0]['containerPort'] == '8888'
        assert system.containers[0].volumes[0] == 'pvc://cloud-top/projects:/work/data'




        



import os
import re
import sys
from shutil import rmtree
from setuptools import setup, find_packages, Command

current = os.path.abspath(os.path.dirname(__file__))
from pip._internal.req import parse_requirements

# parse_requirements() returns generator of pip.req.InstallRequirement objects
install_reqs = parse_requirements(os.path.join(current, "requirements.txt"), session={})

# reqs is a list of requirement
# e.g. ['django==1.5.1', 'mezzanine==1.4.6']
reqs = [str(ir.requirement) for ir in install_reqs]
# def parse_requirements(filename):
#     """ load requirements from a pip requirements file """
#     lineiter = (line.strip() for line in open(filename))
#     return [line for line in lineiter if line and not line.startswith("#")]
# install_reqs = parse_requirements(os.path.join(current, "requirements.txt"))
# requirements = [str(r) for r in install_reqs]

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

with open(os.path.join(current, "tycho", "__init__.py"), encoding="utf8") as f:
    version = re.search(r'__version__="(.*?)"', f.read()).group(1)

class PublishClass(Command):
    description = "Publish the package"
    user_options = []

    # This method must be implemented
    def initialize_options(self):
        pass

    # This method must be implemented
    def finalize_options(self):
        pass

    def run(self):
        try:
            self.status('Removing previous builds…')
            rmtree(os.path.join(current, 'dist'))
            rmtree(os.path.join(current, 'build'))
        except Exception as e:
            print(e)
            pass
        os.system('python setup.py sdist bdist_wheel --universal')
        os.system('twine upload dist/*')
        #os.system('git tag v{0}'.format(about['__version__']))
        #os.system('git push --tags')
        sys.exit()

setup(
    name="tycho-one",
    version=version,
    maintainer="Renci",
    maintainer_email="muralikarthik.k@renci.org",
    description="Tycho is an API, compiler, and executor for cloud native distributed systems.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/helxplatform/tycho.git",
    packages=['tycho', 'tycho.conf', 'tycho.sample', 'tycho.template', 'tycho.test'],
    package_data={'tycho':['*.yaml'],
                  'tycho.conf': ['*.yaml'],
                  'tycho.template': ['*.yaml'],
                  'tycho.test.templates': ['*.yaml']},
    include_package_data=True,
    entry_points={"console_scripts": ["tycho=tycho.api:main"]},
    install_requires=reqs,
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.7',
    cmdclass={
        'publish': PublishClass,
    },
)



import os
import json
import logging
import pytest
import yaml
from tycho.model import System
from tycho.core import Tycho
from tycho.test.lib import system

logger = logging.getLogger (__name__)

def test_pod (system, request):
    print (f"{request.node.name}")
    """ Test generation and integrity of the pod (the deployment template). """
    outputs = system.render ("pod.yaml")
    for output in outputs:
        print (json.dumps (output, indent=2))

        labels = output.get('metadata',{}).get('labels',{})
        pod_name = labels.get('name','')
        tycho_guid = labels.get('tycho-guid',None)
        executor = labels.get('executor','')
    
        assert executor == 'tycho'
        assert pod_name.endswith (tycho_guid)

        containers = output.get('spec',{}).get('containers',{})
        assert containers[0]['name'] == 'jupyter-ds'
        assert containers[0]['image'] == "jupyter/datascience-notebook"
    
def test_service_template (system, request):
    print (f"{request.node.name}")
    """ Verify the generated service selects our pod correctly and is 
        otherwise correctly parameterized.
    """
    pod_specs = system.render ("pod.yaml")
    for pod_spec in pod_specs:
    
        for container in system.containers:
            print (container.name)
            print (system.services)
            service = system.services.get (container.name, None)
            print (service)
            if service:
                logger.debug (f"generating service for container {container.name}")
                service_manifests = system.render (
                    template="service.yaml",
                    context={
                        "service" : service
                    })
                for service_manifest in service_manifests:
                    print (f"{json.dumps(service_manifest,indent=2)}")
                    assert service_manifest['spec']['ports'][0]['port'] == 8888
                    assert service_manifest['spec']['selector']['name'] == system.name
            
                    labels = pod_spec.get('metadata',{}).get('labels',{})
                    assert labels['name'] == service_manifest['spec']['selector']['name']

def test_pvc_template (system, request):
    print (f"{request.node.name}")
    pvcs = system.render ("pvc.yaml")
    for index, pvc in enumerate(pvcs):
        assert pvc.get('spec',{}).get('storageClassName',None) == 'manual'
        assert pvc['spec']['resources']['requests']['storage'] == '2Gi'
        assert pvc['metadata']['labels']['name'] == system.name # Label: Same as the name of the pod.
        assert pvc['metadata']['labels']['tycho-guid'] == system.identifier # Label: Should be same as the pod.
        assert pvc['spec']['volumeName'] == system.volumes[index]['volume_name']

def test_pv_template(system, request):
    print(f"request.node.name")
    pvs = system.render("pv.yaml")
    pvcs = system.render("pvc.yaml")
    for pv_index, pv in enumerate(pvs):
        for pvc_index, pvc in enumerate(pvcs):
            if pv_index == pvc_index:
                assert pv.get("metadata",{}).get("name",None) == pvc.get("spec",{}).get("volumeName",None)
        assert pv.get("metadata",{}).get("name",None) == system.volumes[pv_index]["volume_name"]
        assert pv.get("spec",{}).get("storageClassName",None) == 'manual'
        assert pv.get("spec",{}).get("gcePersistentDisk",{}).get("pdName",None) == system.volumes[pv_index]["disk_name"]
    
def test_networkpolicy (system, request):
    """ Verify the network policy selects our pod, allows our ports, and IP blocks. """
    print (f"{request.node.name}")
    pods = system.render ("pod.yaml")
    for pod in pods:
        pod_labels = pod.get('metadata',{}).get('labels',{})
        guid = pod_labels['tycho-guid']
        policies = system.render ("policy/tycho-default-netpolicy.yaml")
        for policy in policies: 

            found_pod_selector = True
            matched_clients = 0
            for rule in policy['spec']['ingress']:
                for f in rule.get('from', []):
                    selected_guid =  f.get('podSelector',{}).get('matchLabels',{}).get('tycho-guid','')
                    if selected_guid == system.identifier:
                        found_pod_selector = True
                    cidr = f.get('ipBlock',{}).get('cidr','')
                    for name, service in system.services.items ():
                        if cidr in service.clients:
                            matched_clients += 1

            assert policy['spec'].get('egress',None) == None
            assert found_pod_selector == True
            assert matched_clients == len(list(system.services.values())[0].clients)
 
def test_user_defined_template (system, request):
    """ Verify configured template directories are consulted to find templates, that the 
        user defined templates override the system provided templates, and that the contents
        of the user defined template are the ones generated. """
    template_path = os.path.join (os.path.dirname (__file__), "templates")
    system.config['tycho']['templates']['paths'] = [ template_path ]
    spec = system.render ("pod.yaml")
    assert list(spec)[0]['test'] == 'arbitrary_value_for_testing'



import os
import json
import logging
import pytest
import yaml
from tycho.client import TychoClient
from tycho.test.lib import client
from tycho.test.lib import system
from tycho.test.lib import system_request
from unittest import mock

logger = logging.getLogger (__name__)

def test_client_start (mocker, system_request, client, request):
    print (f"{request.node.name}")
    response = {
        "status": "success",
        "result": {
            "name": "jupyter-ds-caa94baea8a849d89e427bd78cad17eb",
            "sid": "caa94baea8a849d89e427bd78cad17eb",
            "containers": {
                "jupyter-datascience": {
                    "ip_address": "127.0.0.1",
                    "port-1": 32661
                }
            }
        },
        "message": "Started system jupyter-ds-caa94baea8a849d89e427bd78cad17eb"
    }
    with mock.patch.object(TychoClient, 'request', return_value=response):
        tycho_system = client.start (system_request)
        result = response['result']
        jupyter = result['containers']['jupyter-datascience']
        assert tycho_system.status == 'success'
        assert tycho_system.name == result['name']
        assert tycho_system.identifier == result['sid']
        assert tycho_system.services[0].ip_address == jupyter['ip_address']
        assert tycho_system.services[0].port == jupyter['port-1']
        assert tycho_system.message == response['message']
